% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{makevars_user}
\alias{makevars_user}
\title{Shim for tools::makevars_user()}
\usage{
makevars_user()
}
\description{
Shim for tools::makevars_user()
}
\keyword{internal}
